@@ -1,8 +1,8 @@
 import re
-from typing import Any, Dict, List, Union
+from typing import Any, Dict, List, Union, Optional
 
 from faker import Faker
-import gender_guesser.detector as gender
+
 import numpy as np
 import pandas as pd
 from simple_colors import green, red
@@ -43,9 +43,13 @@ class FakerGenerator:
     columns_with_assigned_entity: List
     columns_not_synthesized: List
     list_faker: List
+    generation_mark: str
 
     def __init__(
-        self, df_input: Union[str, pd.DataFrame], dict_global_entities: Dict
+        self,
+        df_input: Union[str, pd.DataFrame],
+        dict_global_entities: Dict,
+        generation_mark: Optional[str] = None,
     ) -> "FakerGenerator":
         """
         Create a FakerGenerator instance
@@ -74,6 +78,7 @@ class FakerGenerator:
         self.columns_with_assigned_entity = []
         self.columns_not_synthesized = []
         self.list_faker = []
+        self.generation_mark = generation_mark
 
     def get_columns_with_assigned_entity(self) -> None:
         """
@@ -113,13 +118,23 @@ class FakerGenerator:
         ]
 
         for i in addresses:
-            self.dataset[i] = self.dataset[i].apply(
-                lambda row: (
-                    self.faker.street_address() if not pd.isnull(row) else np.NaN
+            if self.generation_mark == "*":
+                self.dataset[i] = self.dataset[i].apply(
+                    lambda row: (
+                        self.faker.street_address()
+                        if row == self.generation_mark
+                        else row
+                    )
+                )
+                self.list_faker.append(i)
+            else:
+                self.dataset[i] = self.dataset[i].apply(
+                    lambda row: (
+                        self.faker.street_address() if not pd.isnull(row) else np.NaN
+                    )
                 )
-            )
 
-            self.list_faker.append(i)
+                self.list_faker.append(i)
 
     def get_phone_number(self) -> None:
         """
@@ -132,13 +147,23 @@ class FakerGenerator:
         ]
 
         for i in phone_number:
-            self.dataset[i] = self.dataset[i].apply(
-                lambda row: (
-                    self.faker.phone_number() if not pd.isnull(row) else np.NaN
+            if self.generation_mark == "*":
+                self.dataset[i] = self.dataset[i].apply(
+                    lambda row: (
+                        self.faker.phone_number()
+                        if row == self.generation_mark
+                        else row
+                    )
+                )
+                self.list_faker.append(i)
+            else:
+                self.dataset[i] = self.dataset[i].apply(
+                    lambda row: (
+                        self.faker.phone_number() if not pd.isnull(row) else np.NaN
+                    )
                 )
-            )
 
-            self.list_faker.append(i)
+                self.list_faker.append(i)
 
     def get_email_address(self) -> None:
         """
@@ -151,11 +176,21 @@ class FakerGenerator:
         ]
 
         for i in email_address:
-            self.dataset[i] = self.dataset[i].apply(
-                lambda row: (self.faker.free_email() if not pd.isnull(row) else np.NaN)
-            )
+            if self.generation_mark == "*":
+                self.dataset[i] = self.dataset[i].apply(
+                    lambda row: (
+                        self.faker.free_email() if row == self.generation_mark else row
+                    )
+                )
+                self.list_faker.append(i)
+            else:
+                self.dataset[i] = self.dataset[i].apply(
+                    lambda row: (
+                        self.faker.free_email() if not pd.isnull(row) else np.NaN
+                    )
+                )
 
-            self.list_faker.append(i)
+                self.list_faker.append(i)
 
     def get_first_name(self) -> None:
         """
@@ -163,9 +198,6 @@ class FakerGenerator:
 
         """
 
-        detector = gender.Detector(case_sensitive=False)
-        first_name_gender = []
-
         first_name_person = [
             i[0]
             for i in self.columns_with_assigned_entity
@@ -173,52 +205,63 @@ class FakerGenerator:
             and (("first" in i[0].lower()) and ("name" in i[0].lower()))
         ]
 
-        if len(first_name_person) > 0:
-            for col in first_name_person:
-                for name in self.dataset[col]:
-                    if name is not np.NaN:
-                        first_name_gender.append(detector.get_gender(name))
-                    else:
-                        first_name_gender.append("Nan value")
-
-            self.dataset["first_name_gender"] = pd.Series(first_name_gender)
-
         for i in first_name_person:
-            self.dataset[i] = self.dataset.apply(
-                lambda row: (
-                    self.faker.first_name_female()
-                    if (
-                        row["first_name_gender"] == "female"
-                        or row["first_name_gender"] == "mostly_female"
-                    )
-                    else row[i]
-                ),
-                axis=1,
-            )
-            self.dataset[i] = self.dataset.apply(
-                lambda row: (
-                    self.faker.first_name_male()
-                    if (
-                        row["first_name_gender"] == "male"
-                        or row["first_name_gender"] == "mostly_male"
-                    )
-                    else row[i]
-                ),
-                axis=1,
-            )
-            self.dataset[i] = self.dataset.apply(
-                lambda row: (
-                    self.faker.first_name()
-                    if (
-                        row["first_name_gender"] == "unknown"
-                        or row["first_name_gender"] == "andy"
-                    )
-                    else row[i]
-                ),
-                axis=1,
-            )
+            if self.generation_mark == "*":
+                for row in range(0, len(self.dataset[i])):
+                    if self.dataset[i][row] == self.generation_mark:
+                        if (
+                            self.dataset["first_name_gender"][row] == "female"
+                            or self.dataset["first_name_gender"][row] == "mostly_female"
+                        ):
+                            self.dataset[i] = self.dataset[i].apply(
+                                lambda row: (self.faker.first_name_female())
+                            )
+                        elif (
+                            self.dataset["first_name_gender"][row] == "male"
+                            or self.dataset["first_name_gender"][row] == "mostly_male"
+                        ):
+                            self.dataset[i] = self.dataset[i].apply(
+                                lambda row: (self.faker.first_name_male())
+                            )
+                        elif (
+                            self.dataset["first_name_gender"][row] == "unknown"
+                            or self.dataset["first_name_gender"][row] == "andy"
+                        ):
+                            self.dataset[i] = self.dataset[i].apply(
+                                lambda row: (self.faker.first_name())
+                            )
+                        else:
+                            self.dataset[i][row]
 
-            self.list_faker.append(i)
+                self.list_faker.append(i)
+            else:
+                for row in range(0, len(self.dataset[i])):
+                    if not pd.isnull(self.dataset[i][row]):
+                        if (
+                            self.dataset["first_name_gender"][row] == "female"
+                            or self.dataset["first_name_gender"][row] == "mostly_female"
+                        ):
+                            self.dataset[i] = self.dataset[i].apply(
+                                lambda row: (self.faker.first_name_female())
+                            )
+                        elif (
+                            self.dataset["first_name_gender"][row] == "male"
+                            or self.dataset["first_name_gender"][row] == "mostly_male"
+                        ):
+                            self.dataset[i] = self.dataset[i].apply(
+                                lambda row: (self.faker.first_name_male())
+                            )
+                        elif (
+                            self.dataset["first_name_gender"][row] == "unknown"
+                            or self.dataset["first_name_gender"][row] == "andy"
+                        ):
+                            self.dataset[i] = self.dataset[i].apply(
+                                lambda row: (self.faker.first_name())
+                            )
+                        else:
+                            self.dataset[i][row] = np.NaN
+
+                self.list_faker.append(i)
 
         if "first_name_gender" in self.dataset.columns:
             del self.dataset["first_name_gender"]
@@ -238,13 +281,20 @@ class FakerGenerator:
 
         if len(last_name_person) > 0:
             for i in last_name_person:
-                self.dataset[i] = self.dataset[i].apply(
-                    lambda row: (
-                        self.faker.last_name() if not pd.isnull(row) else np.NaN
+                if self.generation_mark == "*":
+                    self.dataset[i] = self.dataset[i].apply(
+                        lambda row: (self.faker.last_name() if row == "*" else row)
                     )
-                )
+                    self.list_faker.append(i)
 
-                self.list_faker.append(i)
+                else:
+                    self.dataset[i] = self.dataset[i].apply(
+                        lambda row: (
+                            self.faker.last_name() if not pd.isnull(row) else np.NaN
+                        )
+                    )
+
+                    self.list_faker.append(i)
 
         else:
             last_name_person = [
@@ -253,13 +303,19 @@ class FakerGenerator:
                 if (("last" in i.lower()) and ("name" in i.lower()))
             ]
             for i in last_name_person:
-                self.dataset[i] = self.dataset[i].apply(
-                    lambda row: (
-                        self.faker.last_name() if not pd.isnull(row) else np.NaN
+                if self.generation_mark == "*":
+                    self.dataset[i] = self.dataset[i].apply(
+                        lambda row: (self.faker.last_name() if row == "*" else row)
+                    )
+                    self.list_faker.append(i)
+                else:
+                    self.dataset[i] = self.dataset[i].apply(
+                        lambda row: (
+                            self.faker.last_name() if not pd.isnull(row) else np.NaN
+                        )
                     )
-                )
 
-                self.list_faker.append(i)
+                    self.list_faker.append(i)
 
     def get_city(self) -> None:
         """
@@ -275,11 +331,19 @@ class FakerGenerator:
         ]
 
         for i in city:
-            self.dataset[i] = self.dataset[i].apply(
-                lambda row: (self.faker.city() if not pd.isnull(row) else np.NaN)
-            )
+            if self.generation_mark == "*":
+                self.dataset[i] = self.dataset[i].apply(
+                    lambda row: (
+                        self.faker.city() if row == self.generation_mark else row
+                    )
+                )
+                self.list_faker.append(i)
+            else:
+                self.dataset[i] = self.dataset[i].apply(
+                    lambda row: (self.faker.city() if not pd.isnull(row) else np.NaN)
+                )
 
-            self.list_faker.append(i)
+                self.list_faker.append(i)
 
     def get_state(self) -> None:
         """
@@ -295,17 +359,39 @@ class FakerGenerator:
 
         for i in state:
             if len(self.dataset[i].iloc[0]) == 2:
-                self.dataset[i] = self.dataset[i].apply(
-                    lambda row: (
-                        self.faker.state_abbr() if not pd.isnull(row) else np.NaN
+                if self.generation_mark == "*":
+                    self.dataset[i] = self.dataset[i].apply(
+                        lambda row: (
+                            self.faker.state_abbr()
+                            if row == self.generation_mark
+                            else row
+                        )
+                    )
+                    self.list_faker.append(i)
+                else:
+                    self.dataset[i] = self.dataset[i].apply(
+                        lambda row: (
+                            self.faker.state_abbr() if not pd.isnull(row) else np.NaN
+                        )
                     )
-                )
-            else:
-                self.dataset[i] = self.dataset[i].apply(
-                    lambda row: self.faker.state() if not pd.isnull(row) else np.NaN
-                )
 
-            self.list_faker.append(i)
+                    self.list_faker.append(i)
+            else:
+                if self.generation_mark == "*":
+                    self.dataset[i] = self.dataset[i].apply(
+                        lambda row: (
+                            self.faker.state() if row == self.generation_mark else row
+                        )
+                    )
+                    self.list_faker.append(i)
+                else:
+                    self.dataset[i] = self.dataset[i].apply(
+                        lambda row: (
+                            self.faker.state() if not pd.isnull(row) else np.NaN
+                        )
+                    )
+
+                    self.list_faker.append(i)
 
     def get_url(self) -> None:
         """
@@ -316,11 +402,19 @@ class FakerGenerator:
         url = [i[0] for i in self.columns_with_assigned_entity if i[1] == "URL"]
 
         for i in url:
-            self.dataset[i] = self.dataset[i].apply(
-                lambda row: (self.faker.url() if not pd.isnull(row) else np.NaN)
-            )
+            if self.generation_mark == "*":
+                self.dataset[i] = self.dataset[i].apply(
+                    lambda row: (
+                        self.faker.url() if row == self.generation_mark else row
+                    )
+                )
+                self.list_faker.append(i)
+            else:
+                self.dataset[i] = self.dataset[i].apply(
+                    lambda row: (self.faker.url() if not pd.isnull(row) else np.NaN)
+                )
 
-            self.list_faker.append(i)
+                self.list_faker.append(i)
 
     def get_zipcode(self) -> None:
         """
@@ -331,11 +425,19 @@ class FakerGenerator:
         zipcode = [i[0] for i in self.columns_with_assigned_entity if i[1] == "ZIPCODE"]
 
         for i in zipcode:
-            self.dataset[i] = self.dataset[i].apply(
-                lambda row: (self.faker.zipcode() if not pd.isnull(row) else np.NaN)
-            )
+            if self.generation_mark == "*":
+                self.dataset[i] = self.dataset[i].apply(
+                    lambda row: (
+                        self.faker.zipcode() if row == self.generation_mark else row
+                    )
+                )
+                self.list_faker.append(i)
+            else:
+                self.dataset[i] = self.dataset[i].apply(
+                    lambda row: (self.faker.zipcode() if not pd.isnull(row) else np.NaN)
+                )
 
-            self.list_faker.append(i)
+                self.list_faker.append(i)
 
     def get_credit_card(self) -> None:
         """
@@ -350,13 +452,25 @@ class FakerGenerator:
         ]
 
         for i in credit_card:
-            self.dataset[i] = self.dataset[i].apply(
-                lambda row: (
-                    self.faker.credit_card_number() if not pd.isnull(row) else np.NaN
+            if self.generation_mark == "*":
+                self.dataset[i] = self.dataset[i].apply(
+                    lambda row: (
+                        self.faker.credit_card_number()
+                        if row == self.generation_mark
+                        else row
+                    )
+                )
+                self.list_faker.append(i)
+            else:
+                self.dataset[i] = self.dataset[i].apply(
+                    lambda row: (
+                        self.faker.credit_card_number()
+                        if not pd.isnull(row)
+                        else np.NaN
+                    )
                 )
-            )
 
-            self.list_faker.append(i)
+                self.list_faker.append(i)
 
     def get_ssn(self) -> None:
         """
@@ -367,11 +481,19 @@ class FakerGenerator:
         ssn = [i[0] for i in self.columns_with_assigned_entity if i[1] == "US_SSN"]
 
         for i in ssn:
-            self.dataset[i] = self.dataset[i].apply(
-                lambda row: (self.faker.ssn() if not pd.isnull(row) else np.NaN)
-            )
+            if self.generation_mark == "*":
+                self.dataset[i] = self.dataset[i].apply(
+                    lambda row: (
+                        self.faker.ssn() if row == self.generation_mark else row
+                    )
+                )
+                self.list_faker.append(i)
+            else:
+                self.dataset[i] = self.dataset[i].apply(
+                    lambda row: (self.faker.ssn() if not pd.isnull(row) else np.NaN)
+                )
 
-            self.list_faker.append(i)
+                self.list_faker.append(i)
 
     def get_country(self) -> None:
         """
@@ -386,11 +508,19 @@ class FakerGenerator:
         ]
 
         for i in country:
-            self.dataset[i] = self.dataset[i].apply(
-                lambda row: (self.faker.country() if not pd.isnull(row) else np.NaN)
-            )
+            if self.generation_mark == "*":
+                self.dataset[i] = self.dataset[i].apply(
+                    lambda row: (
+                        self.faker.country() if row == self.generation_mark else row
+                    )
+                )
+                self.list_faker.append(i)
+            else:
+                self.dataset[i] = self.dataset[i].apply(
+                    lambda row: (self.faker.country() if not pd.isnull(row) else np.NaN)
+                )
 
-            self.list_faker.append(i)
+                self.list_faker.append(i)
 
     def get_columns_not_synthesized(self) -> None:
         """
